/*
OpenIO SDS metautils
Copyright (C) 2014 Worldine, original work as part of Redcurrant
Copyright (C) 2015 OpenIO, modified as part of OpenIO Software Defined Storage

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 3.0 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.
*/

#ifndef OIO_SDS__metautils__lib__metautils_strings_h
# define OIO_SDS__metautils__lib__metautils_strings_h 1

#include <glib.h>

#define STRING_STACKIFY(S) do { \
	gsize _s = strlen((S)); gchar *_t = (S); \
	(S) = alloca(_s + 1); \
	memcpy((S), _t, _s+1); \
	g_free(_t); \
} while (0)

/**
 * Copies in 'd' the part of 's' representing a valid physical namespace.
 *
 * @param d the target buffer to store the physical NS
 * @param s a source string starting with the physical namespace
 * @param dlen the size of the target buffer
 * @return the size of the physical namespace in the source string
 */
gsize metautils_strlcpy_physical_ns(gchar *d, const gchar *s, gsize dlen);

/**
 * @param src
 * @return to be freed with g_free(), not g_strfreev()
 */
gchar ** g_strdupv2(gchar **src);

void metautils_str_reuse(gchar **dst, gchar *src);

void metautils_str_clean(gchar **s);

void metautils_str_replace(gchar **dst, const gchar *src);

const char * metautils_lstrip(register const char *s, register char c);

void metautils_rstrip(register gchar *src, register gchar c);

void metautils_str_upper(register gchar *s);

void metautils_str_lower(register gchar *s);

/* Returns FALSE if 's' is not 'slen' long and contains a non-hexa character. */
gboolean metautils_str_ishexa(const gchar *s, gsize slen);

gsize strlen_len(const guint8 * s, gsize l);

/** Convert an hexa string to its binary form */
gboolean hex2bin(const gchar * src, void * dst, gsize dst_size, GError** error);

/** Fills d (which size is dS) with the hexadecimal alpha-numerical
 * representation of the content of s (which size is sS) */
void buffer2str(const void *s, size_t sS, char *d, size_t dS);

/** Splits the given buffer (considered as a non NULL-terminated) into 
 * newly allocated tokens (wrapping g_strsplit()) */
gchar **buffer_split(const void *buf, gsize buflen, const gchar * separator, gint max_tokens);

/** Check a segment of data is filled with 0 */
gboolean data_is_zeroed(const void *data, gsize data_size);

gchar** metautils_decode_lines(const gchar *start, const gchar *end);

GByteArray* metautils_encode_lines(gchar **strv);

gsize metautils_hash_content_path(const gchar *src, gsize src_size,
	gchar *dst, gsize dst_size, gsize dst_bitlength);

/** Fills the hash_path argument with a hash of the given file_name.
 * The hash is a sequence of hash_depth subdirectories, whose each
 * directory's name is hash_size long. */
void build_hash_path(const char *file_name, int hash_depth, int hash_size,
		char **hash_path);

/** Calls g_strcmp0(a,b) and ignores its third argument. */
int metautils_strcmp3(gconstpointer a, gconstpointer b, gpointer ignored);

/** Returns the boolean value of the textual and human readable boolean
 * string (yes, true, on, yes, 1) */
gboolean metautils_cfg_get_bool(const gchar *value, gboolean def);

gchar ** metautils_strv_append (gchar **, gchar *);

/** Fills 'buf' with buflen random bytes */
void metautils_randomize_buffer(guint8 *bufn, gsize buflen);

/** Frees <p> if p is not NULL */
void g_free0(gpointer p);

/** Frees the first argument and ignores the second */
void g_free1(gpointer p1, gpointer p2);

/** Frees the second argument and ignores the first */
void g_free2(gpointer p1, gpointer p2);

static inline const gchar *
none(const gchar *src)
{
	return src ? src : "null";
}

#endif /*OIO_SDS__metautils__lib__metautils_strings_h*/
